/////////////////////////////////////////////////////////////////////////
///@system  开放平台API接口
///@company 浙江核新同花顺网络信息股份有限公司
///@file    hx_api_struct_common.h
///@brief   定义业务公共数据结构
/////////////////////////////////////////////////////////////////////////
    
#ifndef _HX_API_STRUCT_COMMON_H_
#define _HX_API_STRUCT_COMMON_H_

#include <string.h>

#if defined(_MSC_VER) && _MSC_VER<1600
typedef    signed char              int8_t;
typedef    short                    int16_t;
typedef    int                      int32_t;
typedef    long long                int64_t;
typedef    unsigned char            uint8_t;
typedef    unsigned short           uint16_t;
typedef    unsigned int             uint32_t;
typedef    unsigned long long       uint64_t;
#else
#include <stdint.h>
#endif

#pragma pack(8)

///错误信息的字符串长度
#define HX_ERR_MSG_LEN             128
/// 存放版本号的字符串长度
#define HX_VERSION_LEN             32
/// 可交易日字符串长度 YYYYMMDD
#define HX_TRADING_DAY_LEN         8
/// 存放证券代码的字符串长度
#define HX_SYMBOL_LEN              16
/// 存放证券名称的字符串长度
#define HX_SYMBOL_NAME_LEN         64
/// 本地报单编号的字符串长度
#define HX_LOCAL_ORDER_LEN         64
/// 交易所单号的字符串长度
#define HX_ORDER_EXCH_LEN          32
/// 成交执行编号的字符串长度
#define HX_TRADE_NO_LEN            32
/// 交易所交易员代码字符串长度
#define HX_BRANCH_PBU_LEN          16
/// 用户资金账户的字符串长度
#define HX_ACCOUNT_NAME_LEN        16
/// 定义本地订单号长度
#define HX_LOCAL_ORDER_LEN         64
/// 定义股东号长度
#define HX_STOCK_HOLDER_LEN        15

///响应信息

/////////////////////////////////////////////////////////////////////////
///@brief HX_MESSAGE是 消息类型
/////////////////////////////////////////////////////////////////////////
typedef struct HX_MessageStruct
{
    ///错误代码
    int32_t   code;
    ///错误信息
    char      message[HX_ERR_MSG_LEN];

	HX_MessageStruct()
		: code(0)
	{
		memset(message, 0, sizeof(message));
	}
     
    HX_MessageStruct(int32_t error_code, const char* err_msg)
        : code(error_code)
    {
        memset(message, 0, sizeof(message));
        size_t len = strlen(err_msg);
        if (len>0 && len<HX_ERR_MSG_LEN)
        {
            strncpy(message, err_msg, len);
        }
    }

} HX_MESSAGE;

/////////////////////////////////////////////////////////////////////////
///@brief 版本号类型
/////////////////////////////////////////////////////////////////////////
typedef char HX_VersionType[HX_VERSION_LEN];

/////////////////////////////////////////////////////////////////////////
///@brief HX_LOG_LEVEL是日志输出级别类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_LOG_LEVEL 
{
    HX_LOG_LEVEL_DEBUG	 = 0,    ///=>debug级别, 输出所有日志
	HX_LOG_LEVEL_INFO	 = 1,    ///=>info级别, 输出info及其以上级别的日志
	HX_LOG_LEVEL_WARNING = 2,    ///=>警告级别, 输出警告及其以上级别的日志
	HX_LOG_LEVEL_ERROR	 = 3,    ///=>错误级别, 输出错误级别的日志
	HX_LOG_LEVEL_NOLOG   = 4,	 ///=>关闭日志功能

}HX_LOG_LEVEL;

/////////////////////////////////////////////////////////////////////////
///@brief HX_PROTOCOL_TYPE是通讯传输协议方式
/////////////////////////////////////////////////////////////////////////
typedef enum HX_PROTOCOL_TYPE
{
    HX_PROTOCOL_TCP = 1,    ///=>采用TCP方式传输
    HX_PROTOCOL_UDP = 2     ///=>采用UDP方式传输(暂不支持)
}HX_PROTOCOL_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_EXCHANGE_TYPE是交易所类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_EXCHANGE_TYPE
{
    HX_EXCHANGE_SH = 1,    ///=>上证
    HX_EXCHANGE_SZ = 2,    ///=>深证
    HX_EXCHANGE_ZJ = 3,    ///=>中金
    HX_EXCHANGE_SQ = 4,    ///=>上期
    HX_EXCHANGE_SN = 5,    ///=>上能 /上海能源期货交易所
    HX_EXCHANGE_DL = 6,    ///=>大商所
    HX_EXCHANGE_ZZ = 7,    ///=>郑商所
    HX_EXCHANGE_HK = 8,    ///=>港交所
    HX_EXCHANGE_UNKNOWN    ///=>不存在的交易所类型
}HX_EXCHANGE_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief 供查询的最新信息
/////////////////////////////////////////////////////////////////////////
typedef struct HXSymbolPriceInfo {
    ///交易所代码
    HX_EXCHANGE_TYPE exchange;
    ///证券代码（不包含交易所信息），不带空格，以'\0'结尾
    char symbol[HX_SYMBOL_LEN];
    ///最新价
    double last_price;

	HXSymbolPriceInfo()
		: exchange(HX_EXCHANGE_UNKNOWN)
		, last_price(0.0)
	{
		memset(symbol, 0, sizeof(symbol));
	}
} HXSPI;

/////////////////////////////////////////////////////////////////////////
///@brief HX_PRICE_TYPE是价格类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_PRICE_TYPE
{
    HX_PRICE_LIMIT                = 1,   ///=>限价单 市场:沪/深
    HX_PRICE_BEST5_OR_LIMIT       = 2,   ///=>市价单 最优五档即时成交剩余转限价  市场:沪
    HX_PRICE_BEST5_OR_CANCEL      = 3,   ///=>市价单 最优五档即时成交剩余转撤销  市场:沪/深
    HX_PRICE_BEST_OR_CANCEL       = 4,   ///=>市价单 即时成交剩余转撤销(FAK)     市场:深     
    HX_PRICE_ALL_OR_CANCEL        = 5,   ///=>市价单 全部成交或撤销(FOK)         市场:深
    HX_PRICE_FORWARD_BEST         = 6,   ///=>市价单 本方最优                    市场:深
    HX_PRICE_REVERSE_BEST         = 7,   ///=>市价单 对方最优剩余转限价          市场:深
    HX_PRICE_TYPE_UNKNOWN                ///=>未知或无效价格类型
}HX_PRICE_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_ACTION_TYPE是买卖方向类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_ACTION_TYPE{
    HX_ACTION_TYPE_BUY                   = 1,   ///=>买入 (新股申购/ETF LOF买入/)
    HX_ACTION_TYPE_SELL                  = 2,   ///=>卖出 (逆回购/ETF LOF卖出/)
    HX_ACCOUNT_TYPE_SUBSCRIBE            = 3,   ///=>认购
    HX_ACTION_TYPE_PURCHASE              = 4,   ///=>申购
    HX_ACTION_TYPE_REDEMPTION            = 5,   ///=>赎回
    HX_ACTION_TYPE_UNKNOWN                      ///=>未知买卖方向
}HX_ACTION_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_ORDER_STATUS_TYPE是报单状态类型 0 未报 1 正报 2 已报 3 废单 4 部分成交 5 全部成交 6 部成部撤 7 全部撤单 8 内部撤单
/////////////////////////////////////////////////////////////////////////
typedef enum HX_ORDER_STATUS_TYPE
{
    HX_ORDER_STATUS_TYPE_INIT                  = 0,///=>未报
    HX_ORDER_STATUS_TYPE_IN_PROCESS            = 1,///=>正报
    HX_ORDER_STATUS_TYPE_READY                 = 2,///=>已报
    HX_ORDER_STATUS_TYPE_INVALID               = 3,///=>废单
    HX_ORDER_STATUS_TYPE_PART_TRADED           = 4,///=>部分成交 
    HX_ORDER_STATUS_TYPE_ALL_TRADED            = 5,///=>全部成交
    HX_ORDER_STATUS_TYPE_PART_CANCELED         = 6,///=>部成部撤/部分撤单
    HX_ORDER_STATUS_TYPE_ALL_CANCELED          = 7,///=>全部撤单
    HX_ORDER_STATUS_TYPE_INTERNAL_CANCELED     = 8,///=>内部撤单
    HX_ORDER_STATUS_TYPE_UNKNOWN                   ///=>未知订单状态
}HX_ORDER_STATUS_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_TE_RESUME_TYPE是公有流(订单响应、成交回报)重传方式
/////////////////////////////////////////////////////////////////////////
typedef enum HX_TE_RESUME_TYPE
{
    HX_TERT_TYPE_RESTART = 0,    ///=>从本交易日开始重传
    HX_TERT_TYPE_RESUME  = 1,    ///=>从从上次收到的续传
    HX_TERT_TYPE_QUICK   = 2     ///=>只传送登录后公有流(订单响应、成交回报)
}HX_TE_RESUME_TYPE;

typedef enum HX_SYMBOL_TYPE
{
    HX_SYMBOL_TYPE_STOCK   = 0,   ///=>股票
    HX_SYMBOL_TYPE_BOND    = 1,   ///=>债券
    HX_SYMBOL_TYPE_FUND    = 2,   ///=>基金    
    HX_SYMBOL_TYPE_OPTION  = 3,   ///=>期权
    HX_SYMBOL_TYPE_UNKNOWN        ///=>未知类型
}HX_SYMBOL_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_BUSINESS_TYPE证券业务类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_BUSINESS_TYPE
{
    HX_BUSINESS_TYPE_STOCK                               = 0,   ///=>普通股票业务
    HX_BUSINESS_TYPE_IPO                                 = 1,   ///=>新股申购业务
    HX_BUSINESS_TYPE_ETF                                 = 2,   ///=>ETF 认申赎业务
    HX_BUSINESS_TYPE_LOF                                 = 3,   ///=>LOF 认申赎业务
    HX_BUSINESS_TYPE_MARGIN                              = 4,   ///=>融资融券业务
    HX_BUSINESS_TYPE_ALLOTMENT                           = 5,   ///=>配股配债业务
    HX_BUSINESS_TYPE_STRUCTURED_FUND_PURCHASE_REDEMPTION = 6,   ///=>分级基金认申赎业务
    HX_BUSINESS_TYPE_STRUCTURED_FUND_SPLIT_MERGE         = 7,   ///=>分级基金拆分合并业务
    HX_BUSINESS_TYPE_MONEY_FUND                          = 8,   ///=>货币基金买卖实时申赎业务
    HX_BUSINESS_TYPE_OPTION                              = 9,   ///=>期权业务
    HX_BUSINESS_TYPE_OPTION_EXECUTE                      = 10,  ///=>期权行权
	HX_BUSINESS_TYPE_REPO								 = 11,  ///=>回购业务
    HX_BUSINESS_TYPE_UNKNOWN                                    ///=>未知类型 
}HX_BUSINESS_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_ACCOUNT_TYPE账户类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_ACCOUNT_TYPE
{
    HX_ACCOUNT_TYPE_NORMAL = 0,    ///=>普通账户
    HX_ACCOUNT_TYPE_CREDIT = 1,    ///=>信用账户
    HX_ACCOUNT_TYPE_DERIVE = 2,    ///=>衍生品账户
    HX_ACCOUNT_TYPE_UNKNOWN        ///=>未知账户类型
}HX_ACCOUNT_TYPE;


/////////////////////////////////////////////////////////////////////////
///@brief HX_SPLIT_MERGE_STATUS_TYPE是一个基金当天拆分合并状态类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_SPLIT_MERGE_STATUS_TYPE {
    HX_SPLIT_MERGE_STATUS_TYPE_ALLOW         = 0,    ///=>允许拆分/允许合并
    HX_SPLIT_MERGE_STATUS_TYPE_ONLY_SPLIT    = 1,    ///=>只允许拆分/不允许合并
    HX_SPLIT_MERGE_STATUS_TYPE_ONLY_MERGE    = 2,    ///=>只允许合并/不允许拆分
    HX_SPLIT_MERGE_STATUS_TYPE_FORBIDDEN     = 3,    ///=>不允许拆分/不允许合并
    HX_SPLIT_MERGE_STATUS_TYPE_UNKNOWN               ///=>未知拆分合并状态
}HX_SPLIT_MERGE_STATUS_TYPE;


/////////////////////////////////////////////////////////////////////////
///@brief HX_OPTION_CALL_OR_PUT_TYPE是一个认沽或认购类型 期权类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_OPT_CALL_OR_PUT_TYPE {
    HX_OPTION_TYPE_CALL = 1,        ///=>认购 (看涨)
    HX_OPTION_TYPE_PUT  = 2,        ///=>认沽 (看跌)
    HX_OPTION_TYPE_UNKNOWN          ///=>未知期权类型
}HX_OPTION_CALL_OR_PUT_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_OPTION_EXEC_TYPE是一个行权方式类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_OPTION_EXEC_TYPE {
    HX_OPTION_EXEC_TYPE_EUR = 1,        ///=>欧式
    HX_OPTION_EXEC_TYPE_AME = 2,        ///=>美式
    HX_OPTION_EXEC_TYPE_MIX = 3,        ///=>百慕大
    HX_OPTION_EXEC_TYPE_UNKNOWN         ///=>未知期权行权方式
}HX_OPTION_EXEC_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_POSITION_DIRECTION_TYPE是一个持仓方向类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_POSITION_DIRECTION_TYPE {
    HX_POSITION_DIRECTION_TYPE_NET     = 0,     ///=>净
    HX_POSITION_DIRECTION_TYPE_LONG    = 1,     ///=>多   (期权则为权利方)
    HX_POSITION_DIRECTION_TYPE_SHORT   = 2,     ///=>空   (期权则为义务方)
    HX_POSITION_DIRECTION_TYPE_COVERED = 3,     ///=>备兑 (期权则为备兑义务方)
    HX_POSITION_DIRECTION_TYPE_UNKNOWN          ///=>未知期权持仓头寸方向
}HX_POSITION_DIRECTION_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_TRADE_TYPE是成交类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_TRADE_TYPE{
    HX_TRADE_TYPE_COMMON  = 0,  ///=>普通成交
    HX_TRADE_TYPE_CASH    = 1,  ///=>现金替代
    HX_TRADE_TYPE_UNKNOWN       ///=>未知成交类型
}HX_TRADE_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_ORDER_TYPE是订单类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_ORDER_TYPE{
    HX_ORDER_TYPE_NORMAL       = 1, ///=>普通订单
    HX_ORDER_TYPE_COMBINATION  = 2, ///=>组合订单
    HX_ORDER_TYPE_CONDITION    = 3, ///=>条件订单
    HX_ORDER_TYPE_ALGO         = 4, ///=>算法订单
    HX_ORDER_TYPE_UNKNOWN           ///=>未知订单类型
}HX_ORDER_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_CURRENCY_TYPE是币种类型
/////////////////////////////////////////////////////////////////////////
typedef enum HX_CURRENCY_TYPE{
    HX_CURRENCY_TYPE_CNY    = 0, ///=> 人民币
    HX_CURRENCY_TYPE_USD    = 1, ///=> 美元
    HX_CURRENCY_TYPE_UNKNOWN     ///=> 未知币种类型
}HX_CURRENCY_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_ACCOUNT_TYPE是账号类型
/////////////////////////////////////////////////////////////////////////
//typedef char[HX_ACCOUNT_NAME_LEN] HX_ACCOUNT_TYPE;

/////////////////////////////////////////////////////////////////////////
///@brief HX_STOCK_HOLDER_TYPE是股东号类型
/////////////////////////////////////////////////////////////////////////
///typedef char[HX_STOCK_HOLDER_LEN] HX_STOCK_HOLDER_TYPE;

#pragma pack()

#endif
